@@ -23,7 +23,6 @@
 #ifndef __GINAC_BASIC_H__
 #define __GINAC_BASIC_H__
 
-#include <Python.h>
 #include <cstddef> // for size_t
 #include <vector>
 #include <set>
@@ -38,6 +37,13 @@
 #include "ptr.h"
 #include "assertion.h"
 #include "registrar.h"
+#include "context.h"
+
+// PyObject forward declaration 
+#ifndef PyObject_HEAD
+struct _object;
+typedef _object PyObject;
+#endif
 
 #ifdef PYNAC_HAVE_LIBGIAC
 namespace giac
@@ -50,7 +56,6 @@ namespace GiNaC
 {
 struct ex_is_less;
 }
-using ex_int_map = std::map<GiNaC::ex, int, GiNaC::ex_is_less>;
 #endif
 
 namespace GiNaC {
@@ -62,10 +67,12 @@ class numeric;
 class relational;
 class archive_node;
 class print_context;
+class flint_series_t;
 
 typedef std::vector<ex> exvector;
 typedef std::set<ex, ex_is_less> exset;
 typedef std::map<ex, ex, ex_is_less> exmap;
+using ex_int_map = std::map<GiNaC::ex, int, GiNaC::ex_is_less>;
 
 // Define this to enable some statistical output for comparisons and hashing
 #undef GINAC_COMPARE_STATISTICS
@@ -142,8 +149,8 @@ public:
 	};
 
 	friend class ex;
-	friend struct print_order;
-	friend struct print_order_pair;
+	friend class print_order;
+	friend class print_order_pair;
 	// default constructor, destructor, copy constructor and assignment operator
 protected:
 	basic() : tinfo_key(&tinfo_static), flags(0) {}
@@ -175,11 +182,7 @@ public: // only const functions please (may break reference counting)
 	virtual ex eval(int level = 0) const;
 	virtual ex evalf(int level = 0, PyObject* parent=nullptr) const;
 	virtual ex evalm() const;
-	virtual ex eval_integ() const;
-protected:
-	virtual ex eval_ncmul(const exvector & v) const;
 public:
-	virtual ex eval_indexed(const basic & i) const;
 
 	// printing
 	virtual void print(const print_context & c, unsigned level = 0) const;
@@ -227,7 +230,7 @@ public:
 	virtual bool is_polynomial(const ex & var) const;
 	virtual int degree(const ex & s) const;
 	virtual int ldegree(const ex & s) const;
-	virtual ex coeff(const ex & s, int n = 1) const;
+	virtual ex coeff(const ex & s, const ex & n) const;
 
 	// expand/collect
 	virtual ex expand(unsigned options = 0) const;
@@ -238,6 +241,7 @@ protected:
 	virtual ex derivative(const symbol & s) const;
 public:
 	virtual ex series(const relational & r, int order, unsigned options = 0) const;
+        virtual void useries(flint_series_t& fp, int order) const {}
 
 	// rational functions
 	virtual ex normal(exmap & repl, exmap & rev_lookup, int level = 0, unsigned options = 0) const;
@@ -249,12 +253,6 @@ public:
 	virtual ex smod(const numeric &xi) const;
 	virtual numeric max_coefficient() const;
 
-	// indexed objects
-	virtual exvector get_free_indices() const;
-	virtual ex add_indexed(const ex & self, const ex & other) const;
-	virtual ex scalar_mul_indexed(const ex & self, const numeric & other) const;
-	virtual bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;
-
 	// noncommutativity
 	virtual unsigned return_type() const;
 	virtual tinfo_t return_type_tinfo() const;
@@ -290,6 +288,8 @@ public:
 	ex subs_one_level(const exmap & m, unsigned options) const;
 	ex diff(const symbol & s, unsigned nth = 1) const;
 	bool is_equal(const basic & other) const;
+        bool is_evaluated() const
+            { return global_hold or ((flags & status_flags::evaluated) != 0u); }
 	const basic & hold() const;
 
 	long gethash() const
