@@ -103,7 +103,6 @@ public:
 	ex(int i);
 	ex(unsigned int i);
 	ex(long i);
-	ex(unsigned long i);
 	ex(double const d);
 	ex(PyObject* o);
 
@@ -136,7 +135,6 @@ public:
 	ex eval(int level = 0) const { return bp->eval(level); }
 	ex evalf(int level = 0, PyObject* parent=nullptr) const 
 	{ return bp->evalf(level, parent); }
-	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
 
 	// printing
 	void print(const print_context & c, unsigned level = 0) const;
@@ -192,7 +190,7 @@ public:
 	bool is_polynomial(const ex & vars) const;
 	int degree(const ex & s) const { return bp->degree(s); }
 	int ldegree(const ex & s) const { return bp->ldegree(s); }
-	ex coeff(const ex & s, int n = 1) const { return bp->coeff(s, n); }
+	ex coeff(const ex & s, const ex & n) const { return bp->coeff(s, n); }
 	ex lcoeff(const ex & s) const { return coeff(s, degree(s)); }
 	ex tcoeff(const ex & s) const { return coeff(s, ldegree(s)); }
         void coefficients(const ex & s, expairvec & vec) const;
@@ -236,11 +234,6 @@ public:
         bool is_linear(const symbol& x, ex& a, ex& b) const;
         bool is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const;
 
-	// indexed objects
-	exvector get_free_indices() const { return bp->get_free_indices(); }
-	ex simplify_indexed(unsigned options = 0) const;
-	ex simplify_indexed(const scalar_products & sp, unsigned options = 0) const;
-
 	// domains
 	void set_domain(unsigned d);
 
@@ -271,7 +264,6 @@ private:
 	static basic & construct_from_pyobject(PyObject* o);
 	static basic & construct_from_uint(unsigned int i);
 	static basic & construct_from_long(long i);
-	static basic & construct_from_ulong(unsigned long i);
 	static basic & construct_from_double(double d);
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
@@ -331,12 +323,6 @@ ex::ex(long i) : bp(construct_from_long(i))
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
-inline
-ex::ex(unsigned long i) : bp(construct_from_ulong(i))
-{
-	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-}
-
 inline
 ex::ex(double const d) : bp(construct_from_double(d))
 {
@@ -770,12 +756,6 @@ inline ex series(const ex & thisex, const ex & r, int order, unsigned options =
 inline bool match(const ex & thisex, const ex & pattern, lst & repl_lst)
 { return thisex.match(pattern, repl_lst); }
 
-inline ex simplify_indexed(const ex & thisex, unsigned options = 0)
-{ return thisex.simplify_indexed(options); }
-
-inline ex simplify_indexed(const ex & thisex, const scalar_products & sp, unsigned options = 0)
-{ return thisex.simplify_indexed(sp, options); }
-
 inline ex symmetrize(const ex & thisex)
 { return thisex.symmetrize(); }
 
