@@ -49,8 +49,6 @@
 #include "basic.h"
 #include "constant.h"
 #include "ex.h"
-#include "py_funcs.h"
-#include <factory/factory.h>
 
 #include <gmp.h>
 #include <stdexcept>
@@ -61,6 +59,9 @@
 void ginac_pyinit_Integer(PyObject*);
 void ginac_pyinit_Float(PyObject*);
 void ginac_pyinit_I(PyObject*);
+PyObject* CC_get();
+
+class CanonicalForm;
 
 #ifdef PYNAC_HAVE_LIBGIAC
 namespace giac {
@@ -71,19 +72,19 @@ namespace giac {
 namespace GiNaC {
 
 enum Type {
-//	LONG,
-	DOUBLE=1,
+	LONG=1,
 	PYOBJECT,
 	MPZ,
 	MPQ,
 //	MPFR,
 //	MPFC,
 //	MPQC
-//	LONG,
 };
 
 union Value {
-	double _double;
+        Value() {}
+        Value(signed long int i) : _long(i) {}
+	signed long int _long;
 	mpz_t _bigint;
 	mpq_t _bigrat;
 	PyObject* _pyobject;
@@ -107,10 +108,19 @@ class numeric : public basic {
 	// other constructors
 public:
 	numeric(const numeric&);
-	numeric(int i);
-	numeric(unsigned int i);
-	numeric(long i);
-	numeric(unsigned long i);
+	numeric(int i) : basic(&numeric::tinfo_static), t(LONG), v(i)
+        {
+                hash = (i<0) ? i-1 : i;
+                setflag(status_flags::evaluated | status_flags::expanded);
+        }
+	numeric(unsigned int i) : 
+                basic(&numeric::tinfo_static), t(LONG), v(i), hash(i)
+        { setflag(status_flags::evaluated | status_flags::expanded); }
+	numeric(long i) : basic(&numeric::tinfo_static), t(LONG), v(i)
+        {
+                hash = (i<0) ? i-1 : i;
+                setflag(status_flags::evaluated | status_flags::expanded);
+        }
 	numeric(long numer, long denom);
 	numeric(double d);
 	numeric(mpz_t bigint);
@@ -120,7 +130,14 @@ public:
 	~numeric();
 
 	friend std::ostream& operator<<(std::ostream& os, const numeric& s);
-	friend void coerce(numeric& new_left, numeric& new_right, const numeric& left, const numeric& right);
+	friend void coerce(numeric& new_left, numeric& new_right,
+                        const numeric& left, const numeric& right);
+        friend numeric & operator+=(numeric & lh, const numeric & rh);
+        friend numeric & operator-=(numeric & lh, const numeric & rh);
+        friend numeric & operator*=(numeric & lh, const numeric & rh);
+        friend numeric & operator/=(numeric & lh, const numeric & rh);
+        friend void rational_power_parts(const numeric& a, const numeric& b,
+                        numeric& c, numeric& d, bool& c_unit);
 	// functions overriding virtual functions from base classes
 public:
 
@@ -129,11 +146,12 @@ public:
 	{
 		return 30;
 	}
+	void dbgprint() const override;
 	bool info(unsigned inf) const override;
 	bool is_polynomial(const ex & var) const override;
 	int degree(const ex & s) const override;
 	int ldegree(const ex & s) const override;
-	ex coeff(const ex & s, int n = 1) const override;
+	ex coeff(const ex & s, const ex & n) const override;
 	bool has(const ex &other, unsigned options = 0) const override;
 	ex eval(int level = 0) const override;
 	ex evalf(int level = 0, PyObject* parent = nullptr) const override;
@@ -147,9 +165,11 @@ public:
 	ex to_polynomial(exmap & repl) const override;
 	numeric integer_content() const override;
 	numeric max_coefficient() const override;
-	ex conjugate() const override;
+	numeric conj() const;
+        ex conjugate() const override { return ex(conj()); }
 	ex real_part() const override;
 	ex imag_part() const override;
+        ex series(const relational & r, int order, unsigned options) const override;
 protected:
 
 	/** Implementation of ex::diff for a numeric always returns 0.
@@ -171,7 +191,9 @@ public:
 	const numeric sub(const numeric &other) const;
 	const numeric mul(const numeric &other) const;
 	const numeric div(const numeric &other) const;
-	const numeric power(const numeric &other) const;
+	const ex power(const numeric &other) const;
+	const numeric power(signed long other) const;
+        const numeric pow_intexp(const numeric &exponent) const;
 	const numeric & add_dyn(const numeric &other) const;
 	const numeric & sub_dyn(const numeric &other) const;
 	const numeric & mul_dyn(const numeric &other) const;
@@ -180,15 +202,16 @@ public:
 	const numeric & operator=(int i);
 	const numeric & operator=(unsigned int i);
 	const numeric & operator=(long i);
-	const numeric & operator=(unsigned long i);
 	const numeric & operator=(double d);
 	const numeric & operator=(const numeric& x);
+        void swap(numeric& other);
 	const numeric negative() const;
 	const numeric inverse() const;
 	const numeric step() const;
 	int csgn() const;
 	bool is_equal(const numeric &other) const;
 	bool is_zero() const;
+	bool is_inexact_one() const;
 	bool is_one() const;
 	bool is_minus_one() const;
 	bool is_positive() const;
@@ -218,6 +241,7 @@ public:
 	}
 	long to_long() const;
 	double to_double() const;
+        bool is_long() const { return t == LONG; }
 	bool is_mpz() const { return t == MPZ; }
 	bool is_mpq() const { return t == MPQ; }
         const numeric to_bigint() const;
@@ -228,6 +252,11 @@ public:
         {
                 return t == PYOBJECT;
         }
+        static ex try_py_method(const numeric& x, const std::string& s);
+        static ex try_py_method(const numeric& x1,
+                        const std::string& s,
+                        const numeric& x2);
+        static ex to_dict_parent(const numeric& e, PyObject* dict);
 #ifdef PYNAC_HAVE_LIBGIAC
         giac::gen* to_giacgen(giac::context*) const;
 #endif
@@ -258,7 +287,7 @@ public:
 	const numeric Li2(const numeric &n, PyObject* parent) const;
 	const numeric zeta() const;
 	const numeric stieltjes() const;
-	const numeric lgamma() const;
+	const numeric lgamma(PyObject* parent) const;
 	const numeric tgamma(PyObject* parent) const;
 	const numeric rgamma(PyObject* parent) const;
 	const numeric psi() const;
@@ -282,11 +311,10 @@ public:
 	const numeric iquo(const numeric &b, numeric &r) const;
 	const numeric gcd(const numeric &b) const;
 	const numeric lcm(const numeric &b) const;
-        void factor(std::vector<std::pair<long, int>>& factors) const;
+        void factor(std::vector<std::pair<numeric, int>>& factors, long range=0) const;
+        void factorsmall(std::vector<std::pair<long, int>>& factors, long range=0) const;
         void divisors(std::set<int>& divs) const;
 	
-	int int_length() const;
-
 protected:
 	void print_numeric(const print_context & c, const char *par_open,
 		const char *par_close, const char *imag_sym,
@@ -297,6 +325,8 @@ protected:
 	void do_print_tree(const print_tree & c, unsigned level) const override;
 	void do_print_python_repr(const print_python_repr & c, unsigned level) const override;
 
+        static bool integer_rational_power(numeric& res,
+                const numeric& a, const numeric& b);
 //	numeric operator()(const int& x);
 
 	// member variables
@@ -312,7 +342,6 @@ protected:
 // global constants
 
 extern numeric I;
-extern PyObject *RR, *CC;
 
 // global functions
 
@@ -338,9 +367,9 @@ const numeric Li2(const numeric &x, PyObject* parent=nullptr);
 const numeric Li2(const numeric &x, const numeric &n, PyObject* parent=nullptr);
 const numeric stieltjes(const numeric &x);
 const numeric zeta(const numeric &x);
-const numeric lgamma(const numeric &x);
+const numeric lgamma(const numeric &x, PyObject* parent=nullptr);
 const numeric tgamma(const numeric &x, PyObject* parent=nullptr);
-const numeric rgamma(const numeric &x, PyObject* parent);
+const numeric rgamma(const numeric &x, PyObject* parent=nullptr);
 const numeric psi(const numeric &x);
 const numeric psi(const numeric &n, const numeric &x);
 const numeric factorial(const numeric &n);
@@ -363,7 +392,7 @@ const numeric lcm(const numeric &a, const numeric &b);
 
 // wrapper functions around member functions
 
-inline const numeric pow(const numeric &x, const numeric &y)
+inline ex pow(const numeric &x, const numeric &y)
 {
 	return x.power(y);
 }
